/****************************************************************************
*  Copyright (C) 2012 by Michael Fischer
*
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
****************************************************************************/

#ifndef _DEV_BOARD_H_
#error "Do not include this file directly. Use dev/board.h instead!"
#endif

/*=========================================================================*/
/*  INCLUDE                                                                */
/*=========================================================================*/

/* NiosII */
#include "system.h"
#include "arch/nios2/nios_io.h"
#include "altera_avalon_pio_regs.h"

/* NutOS */
#include <sys/device.h>
#include <dev/dm9000.h>
#include <dev/spibus.h>
#include <arch/nios2/dev/spibus.h>

extern NUTDEVICE devDebug0;
extern NUTSPIBUS spiBus0Nios2;

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/

/*
 * Debug device.
 */
#define DEV_DEBUG             devDebug0
#define DEV_DEBUG_NAME        "uart0"

#define DEV_UART              devDebug0
#define DEV_UART_NAME         "uart0"


/*
 * Ethernet device, is defined by "dev/dm9000.h"
 */
#ifndef NIC_BASE_ADDR
#define NIC_BASE_ADDR         DM9000E_BASE   
#endif
 
#ifndef NIC_DATA_ADDR
#define NIC_DATA_ADDR         (DM9000E_BASE+4)   
#endif

#ifndef NIC_RESET_PORT
#define NIC_RESET_PORT        (DM9000E_BASE+8)
#endif

#ifndef NIC_RESET_PORT_BIT
#define NIC_RESET_PORT_BIT    1
#endif

#ifndef NIC_SIGNAL_IRQ
#define NIC_SIGNAL_IRQ        DM9000E_IRQ
#endif


/*
 * SPI bus
 */ 
#define DEV_SPIBUS            spiBus0Nios2 


/*
 * NVMEM
 */
int DE1ReadData (unsigned int addr, void *buff, size_t siz);
int DE1WriteData (unsigned int addr, CONST void *buff, size_t siz);
 
/*** EOF ***/

